#include "project.h"
#include "stdio.h"
#include "tty_utils.h"
#include "func_list.h"

#define NUM_SEM 128
uint32_t ipcChannel = 0 ;
uint32_t sem_count = NUM_SEM ;
uint32_t mem_ptr[NUM_SEM/32] = { 0u } ;

int current_core = USE_CM0P ;

void do_help(void) ;
#if !spi__DISABLED
void do_spi(void) ;
#endif
#if !gpio_DISABLED
void do_gpio(void) ;
#endif
#if !i2c__DISABLED
void do_i2c(void) ;
#endif
#if !adc__DISABLED
void do_adc(void) ;
#endif
#if !pwm__DISABLED
void do_pwm(void) ;
#endif
void do_mem(void) ;
void do_cm4(void) ;

f_list_type main_func_list[] = {
    { "help", do_help, "show help message" },
    { "mem",  do_mem,  "memory test" },
#if !pwm__DISABLED
    { "pwm",  do_pwm,  "pwm test" },
#endif
#if !adc__DISABLED
    { "adc",  do_adc,  "adc test" },
#endif
#if !i2c__DISABLED
    { "i2c",  do_i2c,  "i2c test" },
#endif
#if !gpio__DISABLED
    { "gpio", do_gpio, "gpio test" },
#endif
#if !spi__DISABLED
    { "spi",  do_spi,  "spi test" },
#endif
    { "cm4",  do_cm4,  "Use the Cortex-M4" },
    { 0,      0,       0} /* list terminator */
} ;

void init_semaphore(void) 
{
    if (CY_IPC_SEMA_SUCCESS != Cy_IPC_Sema_Init(ipcChannel, sem_count, (uint32_t *)mem_ptr)) {
        /* could not allocate semaphore */
        while(1) { } ; /* halt here */
    } 
}

void release_semaphore(void)
{    
    while(Cy_IPC_Sema_Clear(sema_num, false))
        ;
}

int get_semaphore(void)
{
    int result = 0 ;
    if (CY_IPC_SEMA_SUCCESS == Cy_IPC_Sema_Set(sema_num, false)) {
        result = 1 ;
    } else {
        result = 0 ;
    }
    return( result ) ;
}

void init_hardware(void)
{   
    __enable_irq(); /* Enable global interrupts. */ 
    
    init_semaphore() ;
     
    if (get_semaphore()) {
        tty_init(USE_CM0P) ;     
        current_core = USE_CM0P ;
    }
         
    /* Enable CM4.  CY_CORTEX_M4_APPL_ADDR must be updated if CM4 memory layout is changed. */
    Cy_SysEnableCM4(CY_CORTEX_M4_APPL_ADDR); 
}

int main(void)
{
    func_ptr func ;
    
    init_hardware() ;
    
    cls() ;
    splash("MCU Tester (CM0)") ;
    do_help() ;   
    prompt("> ") ;

    for(;;)  {
        if (current_core != USE_CM0P) {
            if (get_semaphore()) {
                current_core = USE_CM0P ;
                tty_init(current_core) ;      
                splash("MCU Tester (CM0)") ;
                do_help() ; 
                prompt("> ") ;
            }
        } else { /* current_core == USE_CM0P */
            if (get_line()) {
                func = get_func(main_func_list, str) ;
                if (func) {
                    func() ;
                } else {
                    print("Unknown Command: ") ;
                    print(str) ;
                    print("\r\n") ;
                    do_help() ;
                }
                if (current_core == USE_CM0P) {
                    prompt("> ") ;
                }
            }
        }   
    }
}

void do_help(void)
{
    show_help(main_func_list) ;
}

void do_cm4(void)
{
    CyDelay(10) ;
    tty_disable(USE_CM0P) ;
    release_semaphore() ;
    current_core = USE_CM4 ; 
    CyDelay(1000) ;
}

