#include "project.h"
#include "stdio.h"
#include "tty_utils.h"
#include "func_list.h"

int current_core = USE_CM0P ;

void do_help(void) ;
#if !spi__DISABLED
void do_spi(void) ;
#endif
#if !gpio_DISABLED
void do_gpio(void) ;
#endif
#if !i2c__DISABLED
void do_i2c(void) ;
#endif
#if !adc__DISABLED
void do_adc(void) ;
#endif
#if !pwm__DISABLED
void do_pwm(void) ;
#endif
void do_mem(void) ;
void do_cm0(void) ;

f_list_type main_func_list[] = {
    { "help", do_help, "show help message" },
    { "mem",  do_mem,  "memory test" },
#if !pwm__DISABLED
    { "pwm",  do_pwm,  "pwm test" },
#endif
#if !adc__DISABLED
    { "adc",  do_adc,  "adc test" },
#endif
#if !i2c__DISABLED
    { "i2c",  do_i2c,  "i2c test" },
#endif
#if !gpio__DISABLED
    { "gpio", do_gpio, "gpio test" },
#endif
#if !spi__DISABLED
    { "spi",  do_spi,  "spi test" },
#endif
    { "cm0", do_cm0, "Use the CorTex-M0+"},
    { 0,      0,       0} /* list terminator */
} ;

void release_semaphore(void)
{
    while(Cy_IPC_Sema_Clear(sema_num, false))
        ;
}

int get_semaphore(void)
{
    int result = 0 ;
    if (CY_IPC_SEMA_SUCCESS == Cy_IPC_Sema_Set(sema_num, false)) {
        result = 1 ;
    } else {
        result = 0 ;
    }
    return( result ) ;
}

void init_hardware(void)
{
    __enable_irq(); /* Enable global interrupts. */ 
}

int main(void)
{
    func_ptr func ;
    
    init_hardware() ;
    
    for(;;) {
        if (current_core != USE_CM4) {
            if (get_semaphore()) {
                current_core = USE_CM4 ;
                tty_init(USE_CM4) ;               
                splash("MCU Tester (CM4)") ;                
                do_help() ;
                prompt("> ") ;
            }
        } else { /* current_core == USE_CM4 */
            if (get_line()) {
                func = get_func(main_func_list, str) ;
                if (func) {
                    func() ;
                } else {
                    print("Unknown Command: ") ;
                    print(str) ;
                    print("\r\n") ;
                    do_help() ;
                }
                if (current_core == USE_CM4) {
                    prompt("> ") ;
                }
            }
        }   
    }
}

void do_help(void)
{
    show_help(main_func_list) ;
}

void do_cm0(void) 
{
    CyDelay(10) ;   
    tty_disable(USE_CM4) ;
    release_semaphore() ;    
    current_core = USE_CM0P ;    
    CyDelay(1000) ;
}

/* [] END OF FILE */
