#include "project.h"
#if !adc__DISABLED
#include "stdio.h"
#include "tty_utils.h"
#include "adc_utils.h"

static int      adc_keep_on  = 1 ;
static uint16_t adc_loop     = 1 ;
static uint16_t adc_interval = 100 ; /* 100 ms */

f_list_type adc_func_list[] = {
    { "help",     do_adc_help,     "show help message" },
    { "loop",     do_adc_loop,     "set loop count"    },
    { "interval", do_adc_interval, "set interval (ms)" },
    { "measure",  do_adc_measure,  "measure adc value" },
    { "status",   do_adc_status,   "print config status" },
    { "run",      do_adc_run,      "measure adc loop times with interval" },
    { "return",   do_adc_return,   "return to the previous mode" },
    { 0,          0,               0} /* list terminator */
} ;

void do_adc(void)
{
    func_ptr func ;
    adc_keep_on = 1 ;
    
    ADC_Start() ;
    
    do_adc_help() ;
                
    prompt("adc> ") ;
    
    while(adc_keep_on) {
        if (get_line()) {
            func = get_func(adc_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_adc_help() ;
            }
            if (adc_keep_on) {
                prompt("adc> ") ;
            }
        }
    }
}


void do_adc_help(void) 
{
    show_help(adc_func_list) ;    
}

void do_adc_return(void) 
{
    adc_keep_on = 0 ;
}

void do_adc_loop(void) 
{
    sscanf(str, "%*s %hd", &adc_loop) ;
}

void do_adc_interval(void) 
{
    sscanf(str, "%*s %hd", &adc_interval) ;
}

void do_adc_measure(void) 
{
    int16_t raw_count, mV ;
    ADC_StartConvert() ;
    ADC_IsEndConversion(CY_SAR_WAIT_FOR_RESULT) ;
    raw_count = ADC_GetResult16(0) ;
    mV = ADC_CountsTo_mVolts(0, raw_count) ;

    if (mV < 0) {
        mV *= -1 ;
        snprintf(str, STR_BUF_LEN, "ADC: -%d.%03d V : ", mV/1000, mV%1000) ;
    } else {
        snprintf(str, STR_BUF_LEN, "ADC:  %d.%03d V : ", mV/1000, mV%1000) ;
    }
    print(str) ;
    snprintf(str, STR_BUF_LEN, "raw count: 0x%04X (%d)\r\n", raw_count&0xFFFF, raw_count) ;
    print(str) ;
}

void do_adc_run(void) 
{
    uint32_t i ;
    for (i = 0 ; i < adc_loop ; i++ ) {
        do_adc_measure() ;
        CyDelay(adc_interval) ;
    }
}

void do_adc_status(void) 
{
    snprintf(str, STR_BUF_LEN, "ADC Config: loop: %d, interval: %d ms\n\r",
        adc_loop, adc_interval) ;
    print(str) ;
}
#endif /* adc__DISABLED */
