#include "project.h"
#if !spi__DISABLED
#include "stdio.h"
#include "tty_utils.h"
#include "spi_utils.h"

static int      spi_keep_on  = 1 ;
static uint16_t spi_loop     = 1 ;
static uint16_t spi_interval = 100 ; /* 100 ms */

f_list_type spi_func_list[] = {
    { "help",     do_spi_help,     "show help message" },
    { "loop",     do_spi_loop,     "set loop count"    },
    { "interval", do_spi_interval, "set interval (ms)" },
    { "write",    do_spi_write,    "write out data"    },
    { "status",   do_spi_status,   "print config status" },
    { "run",      do_spi_run,      "write loop times with interval" },
    { "return",   do_spi_return,   "return to the previous mode" },
    { 0,          0,               0} /* list terminator */
} ;

void do_spi(void) 
{
    func_ptr func ;
    spi_keep_on = 1 ;
    
    SPIM_Start() ;
    
    do_spi_help() ;
    
    prompt("spi> ") ;
    
    while(spi_keep_on) {
        if (get_line()) {
            func = get_func(spi_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_spi_help() ;
            }
            if (spi_keep_on) {
                prompt("spi> ") ;
            }
        }
    }
}

void do_spi_help(void) 
{
    show_help(spi_func_list) ;   
}

void do_spi_return(void) 
{
    spi_keep_on = 0 ;
}

void do_spi_loop(void) 
{
    sscanf(str, "%*s %hd", &spi_loop) ;    
}

void do_spi_interval(void) 
{
    sscanf(str, "%*s %hd", &spi_interval) ;        
}

void do_spi_write(void) 
{
    uint16_t tx_value, rx_value = 0 ;
    sscanf(str, "%*s %hd", &tx_value) ;
    SPIM_Write(tx_value) ;
    while(SPIM_GetNumInTxFifo()) { } /* wait for send completes */
    snprintf(str, STR_BUF_LEN, "SPI: TX: 0x%02X (%d) ",
        tx_value, tx_value) ;
    print(str) ;
    if (SPIM_GetNumInRxFifo()) {
        rx_value = SPIM_Read() ;
        snprintf(str, STR_BUF_LEN, "RX: 0x%02X (%d)", 
            rx_value, rx_value) ;
        print(str) ;
    }
    print("\n\r") ;
}

void do_spi_run(void) 
{
    uint16_t i ;
    uint16_t tx_value, rx_value = 0  ;
    sscanf(str, "%*s %hd", &tx_value) ;
    for (i = 0 ; i < spi_loop ; i++ ) {
        SPIM_Write(tx_value) ;
        while(SPIM_GetNumInTxFifo()) { } /* wait for send completes */
        snprintf(str, STR_BUF_LEN, "SPI: TX: 0x%02X (%d) ",
            tx_value, tx_value) ;
        print(str) ;
        if (SPIM_GetNumInRxFifo()) {
            rx_value = SPIM_Read() ;
            snprintf(str, STR_BUF_LEN, "RX: 0x%02X (%d)", 
                rx_value, rx_value) ;
            print(str) ;
        }
        print("\n\r") ; 
    }
}

void do_spi_status(void) 
{
    snprintf(str, STR_BUF_LEN, "SPI Config: loop: %d, interval: %d ms\n\r",
        spi_loop, spi_interval) ;
    print(str) ;    
}
#endif /* !spi__DISABLED */
